@@ -4,7 +4,6 @@
 @organization: U{IFR 49<http://www.ifr49.org>}
 @license: U{CeCILL version 2<http://www.cecill.info/licences/Licence_CeCILL_V2-en.html>}
 '''
-__docformat__ = "epytext en"
 
 '''
 soma-workflow client classes
@@ -38,6 +37,8 @@ import subprocess
 import sys
 import posixpath
 
+if sys.version_info[:2] >= (2, 6):
+  import json
 #import cProfile
 #import traceback
 
@@ -47,394 +48,17 @@ import soma.workflow.constants as constants
 import soma.workflow.configuration as configuration
 from soma.workflow.errors import TransferError, SerializationError
 
+
 #-------------------------------------------------------------------------------
 # Classes and functions
 #-------------------------------------------------------------------------------
 
-class Job(object):
-  '''
-  Job representation.
-
-  .. note::
-    The command is the only argument required to create a Job.
-    It is also useful to fill the job name for the workflow display in the GUI.
-
-  **command**: *sequence of string or/and FileTransfer or/and SharedResourcePath or/and tuple (relative_path, FileTransfer) or/and sequence of FileTransfer or/and sequence of SharedResourcePath or/and sequence of tuple (relative_path, FileTransfers.)*
-
-    The command to execute. It can not be empty. In case of a shared file system
-    the command is a sequence of string.
-
-    In the other cases, the FileTransfer and SharedResourcePath objects will be
-    replaced by the appropriate path before the job execution.
-
-    The tuples (relative_path, FileTransfer) can be used to refer to a file in a
-    transfered directory.
-
-    The sequences of FileTransfer, SharedResourcePath or tuple (relative_path,
-    FileTransfer) will be replaced by the string "['path1', 'path2', 'path3']"
-    before the job execution. The FileTransfer, SharedResourcePath or tuple
-    (relative_path, FileTransfer) are replaced by the appropriate path inside
-    the sequence.
-
-  **name**: *string*
-    Name of the Job which will be displayed in the GUI
-
-  **referenced_input_files**: *sequence of FileTransfer*
-    List of the FileTransfer which are input of the Job. In other words,
-    FileTransfer which are requiered by the Job to run. It includes the
-    stdin if you use one.
-
-  **referenced_output_files**: *sequence of FileTransfer*
-    List of the FileTransfer which are output of the Job. In other words, the
-    FileTransfer which will be created or modified by the Job.
-
-  **stdin**: *string or FileTransfer or SharedRessourcePath*
-    Path to the file which will be read as input stream by the Job.
-
-  **join_stderrout**: *boolean*
-    Specifies whether the error stream should be mixed with the output stream.
-
-  **stdout_file**: *string or FileTransfer or SharedRessourcePath*
-    Path of the file where the standard output stream of the job will be
-    redirected.
-
-  **stderr_file**: *string or FileTransfer or SharedRessourcePath*
-    Path of the file where the standard error stream of the job will be
-    redirected.
-
-  .. note::
-    Set stdout_file and stderr_file only if you need to redirect the standard
-    output to a specific file. Indeed, even if they are not set the standard
-    outputs will always be available through the WorklfowController API.
-
-  **working_directory**: *string or FileTransfer or SharedRessourcePath*
-    Path of the directory where the job will be executed. The working directory
-    is useful if your Job uses relative file path for example.
-
-  **priority**: *int*
-    Job priority: 0 = low priority. If several Jobs are ready to run at the 
-    same time the jobs with higher priority will be submitted first.
-
-  **parallel_job_info**: *tuple(string, int)*
-    The parallel job information must be set if the Job is parallel (ie. made to
-    run on several CPU).
-    The parallel job information is a tuple: (name of the configuration,
-    maximum number of CPU used by the Job).
-    The configuration name is the type of parallel Job. Example: MPI or OpenMP.
-
-  .. warning::
-    The computing resources must be configured explicitly to use this feature.
-
-  ..
-    **disposal_time_out**: int
-    Only requiered outside of a workflow
-
-  '''
-
-  # sequence of sequence of string or/and FileTransfer or/and SharedResourcePath or/and tuple (relative_path, FileTransfer) or/and sequence of FileTransfer or/and sequence of SharedResourcePath or/and sequence of tuple (relative_path, FileTransfers.)
-  command = None
-
-  # string
-  name = None
-
-  # sequence of FileTransfer
-  referenced_input_files = None
-
-  # sequence of FileTransfer
-  referenced_output_files = None
-
-  # string (path)
-  stdin = None
-
-  # boolean
-  join_stderrout = None
-
-  # string (path)
-  stdout_file = None
-
-  # string (path)
-  stderr_file = None
-
-  # string (path)
-  working_directory = None
-
-  # tuple(string, int)
-  parallel_job_info = None
-
-  # int (in hours)
-  disposal_timeout = None
-
-  # int 
-  priority = None
-
-  def __init__( self,
-                command,
-                referenced_input_files=None,
-                referenced_output_files=None,
-                stdin=None,
-                join_stderrout=False,
-                disposal_timeout=168,
-                name=None,
-                stdout_file=None,
-                stderr_file=None,
-                working_directory=None,
-                parallel_job_info=None,
-                priority=0):
-    if not name:
-      self.name = command[0]
-    else:
-      self.name = name
-    self.command = command
-    if referenced_input_files:
-      self.referenced_input_files = referenced_input_files
-    else: self.referenced_input_files = []
-    if referenced_output_files:
-      self.referenced_output_files = referenced_output_files
-    else: self.referenced_output_files = []
-    self.stdin = stdin
-    self.join_stderrout = join_stderrout
-    self.disposal_timeout = disposal_timeout
-    self.stdout_file = stdout_file
-    self.stderr_file = stderr_file
-    self.working_directory = working_directory
-    self.parallel_job_info = parallel_job_info
-    self.priority = priority
-
-
-class Workflow(object):
-  '''
-  Workflow representation.
-
-  **jobs**: *sequence of Job*
-    Workflow jobs.
-
-  **dependencies**: *sequence of tuple (Job, Job)*
-    Dependencies between the jobs of the workflow.
-    If a job_a needs to be executed before a job_b can run: the tuple
-    (job_a, job_b) must be added to the workflow dependencies. job_a and job_b
-    must belong to workflow.jobs.
-
-  **name**: *string*
-    Name of the workflow which will be displayed in the GUI.
-    Default: workflow_id once submitted
-
-  **root_group**: *sequence of Job and/or Group*
-    Recursive description of the workflow hierarchical structure. For displaying
-    purpose only.
-
-  .. note::
-    root_group is only used to display nicely the workflow in the GUI. It
-    does not have any impact on the workflow execution.
-
-    If root_group is not set, all the jobs of the workflow will be
-    displayed at the root level in the GUI tree view.
-  '''
-  # string
-  name = None
-
-  # sequence of Job
-  jobs = None
-
-  # sequence of tuple (Job, Job)
-  dependencies = None
-
-  # sequence of Job and/or Group
-  root_group = None
-
-  # sequence of Groups built from the root_group
-  groups = None
-
-  # for special user storage
-  user_storage = None
-
-  def __init__(self,
-               jobs,
-               dependencies=None,
-               root_group=None,
-               disposal_timeout=168,
-               user_storage=None,
-               name=None):
-
-    self.name = name
-    self.jobs = jobs
-    if dependencies != None:
-      self.dependencies = dependencies
-    else:
-      self.dependencies = []
-    self.disposal_timeout = 168
-
-    # Groups
-    if root_group:
-      if isinstance(root_group, Group):
-        self.root_group = root_group.elements
-      else:
-        self.root_group = root_group
-      self.groups = []
-      to_explore = []
-      for element in self.root_group:
-        if isinstance(element, Group):
-          to_explore.append(element)
-      while to_explore:
-        group = to_explore.pop()
-        self.groups.append(group)
-        for element in group.elements:
-          if isinstance(element, Group):
-            to_explore.append(element)
-    else:
-      self.root_group = self.jobs
-      self.groups = []
-
-
-
-class Group(object):
-  '''
-  Hierarchical structure of a workflow.
-
-  .. note:
-    It only has a displaying role and does not have any impact on the workflow
-    execution.
-
-  **name**: *string*
-    Name of the Group which will be displayed in the GUI.
-
-  **elements**: *sequence of Job and/or Group*
-    The elements (Job or Group) belonging to the group.
-  '''
-  #string
-  name = None
-
-  #sequence of Job and/or Group
-  elements = None
-
-  def __init__(self, elements, name):
-    '''
-    @type  elements: sequence of Job and/or Group
-    @param elements: the elements belonging to the group
-    @type  name: string
-    @param name: name of the group
-    '''
-    self.elements = elements
-    self.name = name
-
-
-class FileTransfer(object):
-  '''
-  File/directory transfer representation
-
-  .. note::
-    FileTransfers objects are only required if the user and computing resources
-    have a separate file system.
-
-  **client_path**: *string*
-    Path of the file or directory on the user's file system.
-
-  **initial_status**: *constants.FILES_DO_NOT_EXIST or constants.FILES_ON_CLIENT*
-    * constants.FILES_ON_CLIENT for workflow input files
-      The file(s) will need to be transfered on the computing resource side
-    * constants.FILES_DO_NOT_EXIST for workflow output files
-      The file(s) will be created by a job on the computing resource side.
-
-  **client_paths**: *sequence of string*
-    Sequence of path. Files to transfer if the FileTransfers concerns a file
-    series or if the file format involves several associated files or
-    directories (see the note below).
-
-  **name**: *string*
-    Name of the FileTransfer which will be displayed in the GUI.
-    Default: client_path + "transfer"
-
-  .. note::
-    Use client_paths if the transfer involves several associated files and/or
-    directories. Examples:
-
-      * file series
-      * file format associating several file and/or directories
-        (ex: a SPM images are stored in 2 associated files: .img and .hdr)
-        In this case, set client_path to one the files (ex: .img) and
-        client_paths contains all the files (ex: .img and .hdr files)
-
-     In other cases (1 file or 1 directory) the client_paths must be set to None.
-  '''
-
-  # string
-  client_path = None
-
-  # sequence of string
-  client_paths = None
-
-  # constants.FILES_DO_NOT_EXIST constants.FILES_ON_CLIENT
-  initial_status = None
-
-  # int (hours)
-  disposal_timeout = None
-
-  # string
-  name = None
-
-  def __init__( self,
-                is_input,
-                client_path,
-                disposal_timeout = 168,
-                name = None,
-                client_paths = None,
-                ):
-    if name:
-      ft_name = name
-    else:
-      ft_name = client_path + "transfer"
-    self.name = ft_name
-
-    self.client_path = client_path
-    self.disposal_timeout = disposal_timeout
-
-    self.client_paths = client_paths
-
-    if is_input:
-      self.initial_status = constants.FILES_ON_CLIENT
-    else:
-      self.initial_status = constants.FILES_DO_NOT_EXIST
-
-
-class SharedResourcePath(object):
-  '''
-  Representation of path which is valid on either user's or computing resource
-  file system.
-
-  .. note::
-    SharedResourcePath objects are only required if the user and computing
-    resources have a separate file system.
-
-  **namespace**: *string*
-    Namespace for the path. That way several applications can use the same
-    identifiers without risk.
-
-  **uuid**: *string*
-    Identifier of the absolute path.
-
-  **relative_path**: *string*
-    Relative path of the file if the absolute path is a directory path.
-
-  .. warning::
-    The namespace and uuid must exist in the translations files configured on
-    the computing resource side.
-  '''
-
-  relative_path = None
-
-  namespace = None
-
-  uuid = None
-
-  def __init__(self,
-               relative_path,
-               namespace,
-               uuid,
-               disposal_timeout = 168):
-    self.relative_path = relative_path
-    self.namespace = namespace
-    self.uuid = uuid
-    self.disposal_timout = disposal_timeout
-
+# imports required by the users of soma-workflow API (do not remove):
+from soma.workflow.client_types import Job
+from soma.workflow.client_types import Workflow
+from soma.workflow.client_types import Group
+from soma.workflow.client_types import FileTransfer
+from soma.workflow.client_types import SharedResourcePath
 
 
 class WorkflowController(object):
@@ -896,24 +520,16 @@ class WorkflowController(object):
     Deletes the workflow and all its associated elements (FileTransfers and
     Jobs). The worklfow_id will become invalid and can not be used anymore.
     The workflow jobs which are running will be killed.
-    If force is set to True: the client will wait for the workflow to be deleted.
-    If it can't be deleted properly workflow will be deleted from
-    the database server. However, if some jobs are running they won't be kill
-    and will burden the computing resource.
+    If force is set to True: the call will block until the workflow is
+    deleted. With force set to True, if the workflow can not be deleted properly
+    it is deleted from Soma-workflow database. However, if some jobs are still
+    running they are not be killed. In this case the return value is False.
 
     * workflow_id *workflow_identifier*
 
     * force *boolean*
-      If force is set to True, the call won't return before the workflow is
-      deleted. It will wait for the workflow to be deleted.
-      If it can't be deleted properly workflow will be deleted from
-      the database server. However, if some jobs are running they won't be kill
-      and will burden the computing resource (see return value).
 
     * returns: *boolean*
-      If force is True: return True if the running jobs were killed and False
-      if some jobs are possibly still running on the computing resource despite
-      the workflow doesn't exist.
 
     Raises *UnknownObjectError* if the workflow_id is not valid
     '''
@@ -1320,7 +936,8 @@ def _embedded_engine_and_server(config, local_scheduler_config=None):
     from soma.workflow.scheduler import Drmaa
     print "scheduler type: drmaa"
     scheduler = Drmaa(config.get_drmaa_implementation(),
-                      config.get_parallel_job_config())
+                      config.get_parallel_job_config(),
+                      configured_native_spec=config.get_native_specification())
 
   elif config.get_scheduler_type() == 'local_basic':
     from soma.workflow.scheduler import ConfiguredLocalScheduler
@@ -1342,6 +959,62 @@ class Helper(object):
     pass
 
 
+  @staticmethod
+  def list_failed_jobs(workflow_id,
+                      wf_ctrl,
+                      include_aborted_jobs=False,
+                      include_user_killed_jobs=False):
+    '''
+    To spot the problematic jobs in a workflow.
+
+    * workflow_id *workflow identifier*
+    
+    * include_aborted_jobs *boolean*
+      Include the jobs which exit status is constants.EXIT_ABORTED
+
+    * include_user_killed_jobs *boolean*
+      Include the jobs which exit status is constants.USER_KILLED
+
+    * returns: *list of job identifier*
+      Return the list of id of job which status is constants.FAILED
+      or which exit value is not 0.
+    '''
+    (jobs_info, 
+    transfers_info, 
+    workflow_status, 
+    workflow_queue) = wf_ctrl.workflow_elements_status(workflow_id)
+    failed_job_ids = []
+    for (job_id, status, queue, exit_info, dates) in jobs_info:
+      if(status == constants.DONE and exit_info[1] != 0) or \
+        (status == constants.FAILED and \
+        (include_aborted_jobs or exit_info[0] != constants.EXIT_ABORTED) and \
+        (include_user_killed_jobs or exit_info[0] != constants.USER_KILLED)):
+        failed_job_ids.append(job_id)
+    return failed_job_ids
+
+  @staticmethod
+  def delete_all_workflows(wf_ctrl, force=True):
+    '''
+    Delete all the workflows.
+    If force is set to True: the call will block until the workflows are
+    deleted. With force set to True, if a workflow can not be deleted properly
+    it is deleted from Soma-workflow database. However, if some jobs are still
+    running they will not be killed. In this case the return value is False.
+    
+    * wf_ctrl *client.WorkflowController*
+
+    * force *boolean*
+
+    * returns: *boolean*
+    '''
+
+    deleted_properly = True
+    while wf_ctrl.workflows():
+      wf_id = wf_ctrl.workflows().keys()[0]
+      deleted_properly = deleted_properly and  wf_ctrl.delete_workflow(wf_id, force)
+    return deleted_properly
+
+
   @staticmethod
   def wait_workflow(workflow_id,
                     wf_ctrl):
@@ -1432,7 +1105,8 @@ class Helper(object):
   @staticmethod
   def serialize(file_path, workflow):
     '''
-    Saves a workflow to a file => Python Pickle for now.
+    Saves a workflow to a file.
+    Uses JSON format if Python >= 2.6, Python pickle otherwise.
 
     * file_path *String*
 
@@ -1440,17 +1114,30 @@ class Helper(object):
 
     Raises *SerializationError*
     '''
-    try:
-      file = open(file_path, "w")
-      pickle.dump(workflow, file)
-      file.close()
-    except Exception, e:
-      raise SerializationError("%s: %s" %(type(e), e))
+    if sys.version_info[:2] >= (2, 6):
+      try:
+        file = open(file_path, "w")
+        workflow_dict = workflow.to_dict()
+        json.dump(workflow_dict, file, indent=4)
+        file.close()
+      except Exception, e:
+        raise SerializationError("%s: %s" %(type(e), e))
+    else:
+      try:
+        file = open(file_path, "w")
+        pickle.dump(workflow, file)
+        file.close()
+      except Exception, e:
+        raise SerializationError("%s: %s" %(type(e), e))
+  
+
 
   @staticmethod
   def unserialize(file_path):
     '''
-    Loads a workflow from a file 
+    Loads a workflow from a file.
+    Opens JSON format or pickle if Python >= 2.6, only Python pickle otherwise 
+    (see the method: Helper.convert_wf_file_for_p2_5).
 
     * file_path *String*
 
@@ -1458,14 +1145,82 @@ class Helper(object):
 
     Raises *SerializationError*
     '''
-    try:
-      file = open(file_path, "r")
-      workflow = pickle.load(file)
-      file.close()
-    except Exception, e:
-      raise SerializationError("%s: %s" %(type(e), e))
+   
+    if sys.version_info[:2] >= (2, 6):
+      try:
+        file = open(file_path, "r")
+      except Exception, e:
+        raise SerializationError("%s: %s" %(type(e), e))
+
+      workflow = None
+      try:
+        dict_from_json = json.load(file)
+      except ValueError, e:
+        pass
+      else:
+        workflow = Workflow.from_dict(dict_from_json)
+
+      if not workflow:
+        file.close()
+        file = open(file_path, "r")
+        try:
+          workflow = pickle.load(file)
+        except Exception, e:
+          raise SerializationError("%s: %s" %(type(e), e))
+
+      try:
+        file.close()
+      except Exception, e:
+        raise SerializationError("%s: %s" %(type(e), e))
+      
+    else:
+      try:
+        file = open(file_path, "r")
+      except Exception, e:
+        raise SerializationError("%s: %s" %(type(e), e))
+      try: 
+        workflow = pickle.load(file)
+      except Exception, e:
+        raise SerializationError("Error %s: %s \n\n"
+                                 "The workflow file may have been created "
+                                 "using Python >= 2.6 using the JSON format.\n"
+                                 "Use the converter: \n"
+                                 "soma.workflow.client.Helper.convert_wf_file_for_p2_5 "
+                                 " " %(type(e), e))
+      try:
+        file.close()
+      except Exception, e:
+        raise SerializationError("%s: %s" %(type(e), e))
+ 
+
+
+    # compatibility with version 2.2 and previous
+    for job in workflow.jobs:
+      if not hasattr(job, "native_specification"):
+        job.native_specification = None
+
     return workflow
 
+  @staticmethod
+  def convert_wf_file_for_p2_5(origin_file_path, target_file_path):
+    '''
+    This method requires Python >= 2.6.
+    It converts a workflow file created using Python >= 2.6 to workflow file
+    usable in Python 2.5.
+    '''
+    if sys.version_info[:2] < (2, 6):
+      raise Exception("convert_wf_file_for_p2_5 requires Python >= 2.6.")
+  
+    try:
+      o_file = open(origin_file_path, "r")
+      dict_from_json = json.load(o_file)
+      workflow = Workflow.from_dict(dict_from_json)  
+      o_file.close()
+      t_file = open(target_file_path, "w")
+      pickle.dump(workflow, t_file)
+      t_file.close()      
+    except Exception, e:
+      SerializationError("%s: %s" %(type(e), e))
 
   @staticmethod
   def cpu_count():
