@@ -1,7 +1,6 @@
 from __future__ import with_statement 
 
 '''
-@author: Yann Cointepas
 @author: Soizic Laguitton
 @organization: U{IFR 49<http://www.ifr49.org>}
 @license: U{CeCILL version 2<http://www.cecill.info/licences/Licence_CeCILL_V2-en.html>}
@@ -35,7 +34,6 @@ import soma.workflow.constants as constants
 from soma.workflow.client import FileTransfer
 from soma.workflow.errors import UnknownObjectError, DatabaseError
 
-__docformat__ = "epytext en"
 
 #-----------------------------------------------------------------------------
 # Globals and constants
@@ -329,7 +327,8 @@ class WorkflowDatabaseServer( object ):
         except Exception, e:
           cursor.close()
           connection.close()
-          raise DatabaseError("Your database file might not be compatible "
+          raise DatabaseError(str(e) + "\n\n"
+                              "Your database file might not be compatible "
                               "with the current version of Soma-workflow.\n\n"
                               "To solve the problem: \n  1. Log on the host"
                               " " + repr(socket.gethostname()) + " (if it is not the current machine). \n  2. Delete"
@@ -393,7 +392,6 @@ class WorkflowDatabaseServer( object ):
     by valid job.
     '''
     self.logger.debug("=> clean")
-    self.remove_non_registered_files()
     with self._lock:
       connection = self._connect()
       cursor = connection.cursor()
@@ -457,6 +455,8 @@ class WorkflowDatabaseServer( object ):
       cursor.close()
       connection.commit()
       connection.close()
+
+      #self.remove_non_registered_files()
      
   def remove_non_registered_files(self):
     self.logger.debug("=> remove_non_registered_files")
