@@ -106,10 +106,13 @@ class Drmaa(Scheduler):
   
   logger = None
 
+  _configured_native_spec = None
+
   def __init__(self, 
                drmaa_implementation, 
                parallel_job_submission_info, 
-               tmp_file_path=None):
+               tmp_file_path=None, 
+               configured_native_spec=None):
 
     self.logger = self.logger = logging.getLogger('ljp.drmaajs')
 
@@ -127,6 +130,8 @@ class Drmaa(Scheduler):
 
     self.parallel_job_submission_info = parallel_job_submission_info
 
+    self._configured_native_spec = configured_native_spec 
+
     self.logger.debug("Parallel job submission info: %s", 
                       repr(parallel_job_submission_info))
 
@@ -200,7 +205,7 @@ class Drmaa(Scheduler):
     else:
       command = job_command
 
-    #self.logger.debug("command: " + repr(command))
+    self.logger.debug("command: " + repr(command))
     
     stdout_file = job.plain_stdout()
     stderr_file = job.plain_stderr()
@@ -238,10 +243,25 @@ class Drmaa(Scheduler):
       if working_directory:
         self._drmaa.setAttribute(drmaaJobId, "drmaa_wd", working_directory)
 
-      if job.queue:
+      self.logger.debug("JOB NATIVE_SPEC " + repr(job.native_specification))
+      self.logger.debug("CONFIGURED NATIVE SPEC " + repr(self._configured_native_spec))
+      native_spec = None
+      if job.native_specification:
+        native_spec = job.native_specification
+      elif self._configured_native_spec:
+        native_spec = self._configured_native_spec
+      
+      if job.queue and native_spec:
+        self._drmaa.setAttribute(drmaaJobId, "drmaa_native_specification", "-q " + str(job.queue) + " " + str(native_spec))
+        self.logger.debug("NATIVE specification " + "-q " + str(job.queue) + " " + str(native_spec))
+      elif job.queue:
         self._drmaa.setAttribute(drmaaJobId, "drmaa_native_specification", "-q " + str(job.queue))
+        self.logger.debug("NATIVE specification " + "-q " + str(job.queue))
+      elif native_spec:
+        self._drmaa.setAttribute(drmaaJobId, "drmaa_native_specification", str(native_spec))
+        self.logger.debug("NATIVE specification " + str(native_spec))
+
 
-      #self._drmaa.setAttribute(drmaaJobId, "drmaa_native_specification", "-l h_rt=0:0:30" )
       
       if job.parallel_job_info :
         parallel_config_name, max_node_number = job.parallel_job_info
@@ -254,6 +274,12 @@ class Drmaa(Scheduler):
       drmaaSubmittedJobId = self._drmaa.runJob(drmaaJobId)
       self._drmaa.deleteJobTemplate(drmaaJobId)
     except DrmaaError, e:
+      try:
+        f = open(stderr_file, "wa")
+        f.write("Error in job submission: %s" %(e))
+        f.close()
+      except IOError, ioe:
+        pass
       self.logger.error("Error in job submission: %s" %(e))
       raise DRMError("Job submission error: %s" %(e))
 
@@ -457,12 +483,12 @@ class LocalScheduler(Scheduler):
       del self._processes[job_id]
 
     # run new jobs
-    while self._queue and \
-          len(self._processes) < self._proc_nb:
+    while (self._queue and
+           len(self._processes) < self._proc_nb):
       job_id = self._queue.pop(0)
       job = self._jobs[job_id]
       #print "new job " + repr(job.job_id)
-      process = self._create_process(job)
+      process = LocalScheduler.create_process(job)
       if process == None:
         self._exit_info[job.job_id] = (constants.EXIT_ABORTED,
                                    None,
@@ -473,7 +499,14 @@ class LocalScheduler(Scheduler):
         self._processes[job.job_id] = process
         self._status[job.job_id] = constants.RUNNING
 
-  def _create_process(self, engine_job):
+
+  @staticmethod
+  def create_process(engine_job):
+    '''
+    * engine_job *EngineJob*
+   
+    * returns: *Subprocess process* 
+    '''
 
     command = engine_job.plain_command()
 
