@@ -33,11 +33,11 @@ class WorkflowExamples(object):
   
   @staticmethod
   def get_workflow_example_list():
-    return ["simple", "multiple", "with exception 1", "with exception 2", "command check test", "special transfers", "hundred of jobs", "ten jobs", "fake pipelineT1", "serial", "hundred with dependencies", "thousands", "thousands with dependencies"]
+    return ["simple", "multiple", "with exception 1", "with exception 2", "command check test", "special transfers", "hundred of jobs", "ten jobs", "fake pipelineT1", "serial", "hundred with dependencies", "thousands", "thousands with dependencies", "native specification for PBS", "wrong native specification for PBS"]
   
-  def __init__(self, with_tranfers, with_shared_resource_path = False):
+  def __init__(self, with_transfers, with_shared_resource_path = False):
     '''
-    @type with_tranfers: boolean
+    @type with_transfers: boolean
     @type with_shared_resource_path: boolean
 
     if with_transfer and not with_shared_resource_path:
@@ -63,7 +63,7 @@ class WorkflowExamples(object):
     if not os.path.isdir(self.output_dir):
       os.mkdir(self.output_dir)
        
-    self.with_transfers = with_tranfers
+    self.with_transfers = with_transfers
     self.with_shared_resource_path = with_shared_resource_path
     
     # local path
@@ -185,21 +185,21 @@ class WorkflowExamples(object):
   def get_workflow_example(self, example_index):
     workflow = None
     if example_index == 0:
-      workflow = self.simpleExample()
+      workflow = self.simple_example()
     elif example_index == 1:
-      workflow = self.multipleSimpleExample()
+      workflow = self.multiple_simple_example()
     elif example_index == 2:
-      workflow = self.simpleExampleWithException1()
+      workflow = self.simple_example_with_exception1()
     elif example_index == 3:
-      workflow = self.simpleExampleWithException2()
+      workflow = self.simple_example_with_exception2()
     elif example_index == 4:
-      workflow = self.command_test()
+      workflow = self.special_command()
     elif example_index == 5:
-      workflow = self.special_transfer_test()
+      workflow = self.special_transfer()
     elif example_index == 6:
-      workflow = self.n_jobs(500)
+      workflow = self.n_jobs(100)
     elif example_index == 7:
-      workflow = self.n_jobs(50)
+      workflow = self.n_jobs(10)
     elif example_index == 8:
       workflow = self.fake_pipelineT1()
     elif example_index == 9:
@@ -210,29 +210,33 @@ class WorkflowExamples(object):
       workflow = self.n_jobs(12000)
     elif example_index == 12:
       workflow = self.n_jobs_with_dependencies(2000)
+    elif example_index == 13:
+      workflow = self.native_spec_pbs()
+    elif example_index == 14:
+      workflow = self.wrong_native_spec_pbs()
     return workflow
 
-  def job1(self):
+  def job1(self, option=None):
     if self.with_transfers and not self.with_shared_resource_path: 
       job1 = Job(["python", self.tr_script1, self.tr_file0,  self.tr_file11, self.tr_file12, "20"], 
                   [self.tr_file0, self.tr_script1, self.tr_stdin1], 
                   [self.tr_file11, self.tr_file12], 
-                  self.tr_stdin1, False, 168, "job1")
+                  self.tr_stdin1, False, 168, "job1", native_specification=option)
     elif self.with_transfers and self.with_shared_resource_path:
       job1 = Job( ["python", self.sh_script1, self.sh_file0,  self.tr_file11, self.tr_file12, "20"], 
                   [], 
                   [self.tr_file11, self.tr_file12], 
-                  self.sh_stdin1, False, 168, "job1")
+                  self.sh_stdin1, False, 168, "job1", native_specification=option)
     elif not self.with_transfers and self.with_shared_resource_path:
       job1 = Job( ["python", self.sh_script1, self.sh_file0,  self.sh_file11, self.sh_file12, "20"], 
                   None, 
                   None, 
-                  self.sh_stdin1, False, 168, "job1")
+                  self.sh_stdin1, False, 168, "job1", native_specification=option)
     else:
       job1 = Job( ["python", self.lo_script1, self.lo_file0,  self.lo_file11, self.lo_file12, "20"], 
                   None, 
                   None, 
-                  self.lo_stdin1,  False, 168, "job1")
+                  self.lo_stdin1,  False, 168, "job1", native_specification=option)
     return job1
     
   def job2(self):
@@ -418,7 +422,7 @@ class WorkflowExamples(object):
     
     return job
     
-  def special_transfer_test(self):
+  def special_transfer(self):
      # jobs
     test_dir_contents = self.job_test_dir_contents()
     test_multi_file_format = self.job_test_multi_file_format()
@@ -432,7 +436,7 @@ class WorkflowExamples(object):
     return workflow
       
 
-  def command_test(self):
+  def special_command(self):
     
     # jobs
     test_command_job = self.job_test_command_1()
@@ -446,7 +450,7 @@ class WorkflowExamples(object):
     
     return workflow
       
-  def simpleExample(self):
+  def simple_example(self):
     
     # jobs
     job1 = self.job1()
@@ -468,8 +472,37 @@ class WorkflowExamples(object):
     workflow = Workflow(jobs, dependencies, root_group=[group_2, job4])
     
     return workflow
+
+  def wrong_native_spec_pbs(self):
+     # jobs
+    job1 = self.job1(option="-l walltime=5:00:00, pmem=16gb")
+    job2 = self.job1(option="-l walltime=5:00:0")
+    job3 = self.job1()
+    
+    #building the workflow
+    jobs = [job1, job2, job3]
+    
+    workflow = Workflow(jobs, dependencies=[], name="jobs with wrong native spec for pbs")
+    
+    return workflow
+
+
+  def native_spec_pbs(self):
+     # jobs
+    job1 = self.job1(option="-l walltime=5:00:00,pmem=16gb")
+    job2 = self.job1(option="-l walltime=5:00:0")
+    job3 = self.job1()
+    
+    #building the workflow
+    jobs = [job1, job2, job3]
+    
+    workflow = Workflow(jobs, dependencies=[], name="jobs with native spec for pbs")
+    
+    return workflow
+
+ 
   
-  def simpleExampleWithException1(self):
+  def simple_example_with_exception1(self):
                                                           
     # jobs
     if self.with_transfers and not self.with_shared_resource_path:
@@ -513,7 +546,7 @@ class WorkflowExamples(object):
     return workflow
   
   
-  def simpleExampleWithException2(self):
+  def simple_example_with_exception2(self):
    
     # jobs
     job1 = self.job1()
@@ -558,10 +591,10 @@ class WorkflowExamples(object):
     return workflow
   
   
-  def multipleSimpleExample(self):
-    workflow1 = self.simpleExample()
-    workflow2 = self.simpleExampleWithException1()
-    workflow3 = self.simpleExampleWithException2()
+  def multiple_simple_example(self):
+    workflow1 = self.simple_example()
+    workflow2 = self.simple_example_with_exception1()
+    workflow3 = self.simple_example_with_exception2()
     
     jobs = workflow1.jobs
     jobs.extend(workflow2.jobs)
@@ -701,82 +734,88 @@ class WorkflowExamples(object):
 
     return workflow
 
+class WorkflowTest(unittest.TestCase):
+  
+  LOCAL_PATH = "local path"
+  FILE_TRANSFER = "file transfer"
+  SHARED_RESOURCE_PATH = "shared resource path"
 
+  wf_ctrl = None
 
-class WfTest(unittest.TestCase):
-  '''
-  Abstract class for soma-workflow workflow test. 
-  '''
-  wf = None
+  path_management = None
+
+  wf_examples = None
 
   wf_id = None
 
-  wf_ctrl = None
-  
-  wf_ex = None
+  @classmethod
+  def setup_wf_controller(cls, workflow_controller):
+    cls.wf_ctrl = workflow_controller
 
-  @staticmethod
-  def setup_wf_controller(workflow_controller):
-    WfTest.wf_ctrl = workflow_controller
+  @classmethod
+  def setup_path_management(cls, path_management):
+    '''
+    * path_management: LOCAL_PATH, FILE_TRANSFER or SHARED_RESOURCE_PATH
+    '''
+    cls.path_management = path_management
 
-  def setUp(self):
-    raise Exception('WfTest is an abstract class. SetUp must be implemented in subclass')
+  def setUp(self):  
+    if WorkflowTest.path_management == WorkflowTest.LOCAL_PATH:
+      WorkflowTest.wf_examples = WorkflowExamples(with_transfers=False,
+                                          with_shared_resource_path=False)
+    elif WorkflowTest.path_management == WorkflowTest.FILE_TRANSFER: 
+      WorkflowTest.wf_examples = WorkflowExamples(with_transfers=True,
+                                          with_shared_resource_path=False)
+    elif WorkflowTest.path_management == WorkflowTest.SHARED_RESOURCE_PATH:
+      WorkflowTest.wf_examples = WorkflowExamples(with_transfers=False,
+                                          with_shared_resource_path=True)
+    #raise Exception("WorkflowTest is an abstract class.")
 
   def tearDown(self): 
     if self.wf_id:
-      WfTest.wf_ctrl.delete_workflow(self.wf_id)
-    self.assertRaises(UnknownObjectError, WfTest.wf_ctrl.workflow, self.wf_id)
+      WorkflowTest.wf_ctrl.delete_workflow(self.wf_id)
 
-    
-  #def test_workflows(self):
-    #res = WfTest.wf_ctr.workflows.keys()
-    #self.failUnless(self.wf_id in res)
-    
-  #def test_stop(self):
-    ##TBI
-    #pass
 
-  #def test_restart(self):
-    ##TBI
-    #pass
+class MultipleTest(WorkflowTest):
 
   def test_result(self):
-    raise Exception('WfTest is an abstract class. test_result must be implemented in subclass')
+    workflow = WorkflowTest.wf_examples.multiple_simple_example() 
+    self.wf_id = WorkflowTest.wf_ctrl.submit_workflow(workflow=workflow, 
+                                             name="unit test multiple")
+    if self.path_management == WorkflowTest.FILE_TRANSFER:
+      Helper.transfer_input_files(self.wf_id, WorkflowTest.wf_ctrl)
 
-class Simple_Shared(WfTest):
+    Helper.wait_workflow(self.wf_id, WorkflowTest.wf_ctrl)
+    status = self.wf_ctrl.workflow_status(self.wf_id)
+    
+    self.assert_(status == constants.WORKFLOW_DONE)
+    self.assert_(len(Helper.list_failed_jobs(self.wf_id, 
+                                             WorkflowTest.wf_ctrl)) == 2)
+    self.assert_(len(Helper.list_failed_jobs(self.wf_id, 
+                                             WorkflowTest.wf_ctrl,
+                                             include_aborted_jobs=True)) == 6)
+    # TODO: check the stdout and stderrr
 
-  def setUp(self):
-    workflow_examples = WorkflowExamples(with_tranfers=False,
-                                         with_shared_resource_path=False)
-    self.wf_ex = workflow_examples
-    self.wf = self.wf_ex.simpleExample() 
-    self.wf_id = WfTest.wf_ctrl.submit_workflow(workflow=self.wf, 
-                                              name="unit test multiple")
+class SpecialCommandTest(WorkflowTest):
 
   def test_result(self):
-    Helper.wait_workflow(self.wf_id, WfTest.wf_ctrl)
-    status = WfTest.wf_ctrl.workflow_status(self.wf_id)
+    workflow = WorkflowTest.wf_examples.special_command() 
+    self.wf_id = WorkflowTest.wf_ctrl.submit_workflow(workflow=workflow, 
+                                             name="unit test command check")
+    if self.path_management == WorkflowTest.FILE_TRANSFER:
+      Helper.transfer_input_files(self.wf_id, WorkflowTest.wf_ctrl)
+
+    Helper.wait_workflow(self.wf_id, WorkflowTest.wf_ctrl)
+    status = self.wf_ctrl.workflow_status(self.wf_id)
     
-    self.failUnless(status == constants.WORKFLOW_DONE)
-
-
-class Multiple(WfTest):
-  
-  def setUp(self):
-    workflow_examples = WorkflowExamples(False)
-    self.wf_ex = workflow_examples
-    self.wf = self.wf_ex.simpleExample() #multipleSimpleExample()
-    self.wf_id = WfTest.wf_ctrl.submit_workflow(workflow=self.wf, 
-                                                name="unit test multiple")
-
-  def test_result(self):
-    Helper.wait_workflow(self.wf_id, WfTest.wf_ctrl)
-    status = WfTest.wf_ctrl.workflow_status(self.wf_id)
-    
-    self.failUnless(status == constants.WORKFLOW_DONE)
-    
-
-
+    self.assert_(status == constants.WORKFLOW_DONE)
+    self.assert_(len(Helper.list_failed_jobs(self.wf_id, 
+                                             WorkflowTest.wf_ctrl)) == 0)
+    self.assert_(len(Helper.list_failed_jobs(self.wf_id, 
+                                             WorkflowTest.wf_ctrl,
+                                             include_aborted_jobs=True)) == 0)
+    # TODO: check the stdout 
+ 
 
 if __name__ == '__main__':
   
@@ -815,7 +854,7 @@ if __name__ == '__main__':
   sys.stdout.write("---------------------------------\n")
   
   # Workflow types
-  wf_types = ["multiple", "command check test", "special transfers", "ten jobs"]
+  wf_types = ["multiple", "special command"]
   sys.stdout.write("Workflow example to test: \n")
   sys.stdout.write("all -> all \n")
   for i in range(0, len(wf_types)):
@@ -838,7 +877,9 @@ if __name__ == '__main__':
 
   # File path type 
   sys.stdout.write("---------------------------------\n")
-  path_types = ["local path", "shared resource path", "file transfer"]
+  path_types = [WorkflowTest.LOCAL_PATH, 
+                WorkflowTest.FILE_TRANSFER,
+                WorkflowTest.SHARED_RESOURCE_PATH]
   sys.stdout.write("File path type: \n")
   sys.stdout.write("all -> all \n")
   for i in range(0, len(path_types)):
@@ -884,24 +925,35 @@ if __name__ == '__main__':
   sys.stdout.write("---------------------------------\n")
 
   wf_controller = WorkflowController(resource_id, login, password)
-  WfTest.setup_wf_controller(wf_controller)
+  WorkflowTest.setup_wf_controller(wf_controller)
 
-  suite_list =  []
-  tests = selected_test_type
-  wf_types = ["multiple", "command check test", "special transfers", "ten jobs"]
 
-  if "multiple" in selected_wf_type:
-    suite_list.append(unittest.TestSuite(map(Multiple, tests)))
-  if "command check test" in selected_wf_type:
-    suite_list.append(unittest.TestSuite(map(CommandCheck, tests)))
-  if "special transfers" in selected_wf_type:
-    suite_list.append(unittest.TestSuite(map(SpecialTransfers, tests)))
-  if "ten jobs" in selected_wf_type:
-    suite_list.append(unittest.TestSuite(map(TenJobs, tests)))
+  test_results = {}
+
+  for path_management in selected_path_type:
+    print "\n**********************************************************************"
+    print "Test Suite: " + repr(path_management) + ":\n"
+    WorkflowTest.setup_path_management(path_management)
+
+    suite_list =  []
+    tests = selected_test_type
+
+    if "multiple" in selected_wf_type:
+      suite_list.append(unittest.TestSuite(map(MultipleTest, 
+                                               selected_test_type)))
+    if "special command" in selected_wf_type:
+      suite_list.append(unittest.TestSuite(map(SpecialCommandTest, 
+                                               selected_test_type)))
+
+    alltests = unittest.TestSuite(suite_list)
+    test_result = unittest.TextTestRunner(verbosity=2).run(alltests)
+    test_results[path_management] = test_result
+
+  if len(test_results) > 1:
+    for path_management, test_result in test_results.iteritems():
+      if not test_result.wasSuccessful():
+        print repr(path_management) + " test suite failed !"
 
-  alltests = unittest.TestSuite(suite_list)
-  unittest.TextTestRunner(verbosity=2).run(alltests)
-  
   sys.exit(0)
 
 
